Program TitleMD;
  {Program for sending text file with titles of MD and tracks to MD
  through IR-LED connected to LPR port (through generator 38kHz - see Fig.)}

uses
  Crt,MDChars,MDChars2,InfraRed,CS_Conv;

const
  CSCodePage:tCode=Win1250;     {Used Code Page for Czech language}

type
  tDirect=(Left,Right,All);

var
  F:Text;
  S:String;
  I:Integer;
  TrackNo:Byte;
  Title:String;
  LastTrackNo:Integer;
  MyConvertTable:tConvertTable;

function ConvTab2Space(S:String):String;
  var
    I:Integer;
  begin
    for I:=1 to Length(S) do
      if S[I]=#9 then
        S[I]:=' ';
    ConvTab2Space:=S;
  end;

function TrimSpace(S:String;Direct:tDirect):String;
  var
    Pa1,Pa2:Byte;
    Spom:String;
  begin
    Pa1:=1;
    Pa2:=Length(S);
    if (Direct=Left) or (Direct=All) then
      while (S[Pa1]=' ') and (Pa1<Pa2) do
        Inc(Pa1);
    if (Direct=Right) or (Direct=All) then
      while (S[Pa2]=' ') and (Pa2>Pa1) do
        Dec(Pa2);
    Spom:=Copy(S,Pa1,Pa2-Pa1+1);
    if Spom=' ' then
      Spom:='';
    TrimSpace:=Spom;
  end;

procedure WriteLine;
  begin
    WriteLn('-------------------------------------------------------------------------------');
  end;

procedure ReadIniFile;
  var
    I:Integer;
    LX:LongInt;
    Code:Integer;
    LineNo:Integer;
    S,S1,S2:String;
    ErrDetected:Boolean;

  procedure WriteErr;
    begin
      WriteLn('Error in ini file - on line: ',LineNo);
      ErrDetected:=True;
    end;
  function UpString(S:String):String;
    var
      I:Integer;
    begin
      for I:=1 to Length(S) do
        S[I]:=UpCase(S[I]);
      UpString:=S;
    end;

  begin
    {$I-}
    Assign(F,'TitleMD.ini');
    Reset(F);
    if IOResult<>0 then
      begin
        WriteLn('Warning: Ini file "TitleMD.ini" not found - using default values.');
        WriteLine;
        Exit;
      end;
    {$I+}
    ErrDetected:=False;
    LineNo:=0;
    while not Eof(F) do
      begin
        ReadLn(F,S);
        Inc(LineNo);
        S:=TrimSpace(S,All);
        S:=UpString(S);
        {$B-}
        if (S<>'') and (S[1]<>';') then
          begin
            I:=Pos('=',S);
            if I=0 then
              WriteErr
            else
              begin
                S1:=TrimSpace(Copy(S,1,I-1),All);
                S2:=TrimSpace(Copy(S,I+1,255),All);
                if S1=UpString('CSCodePage') then
                  begin
                    if S2=UpString('Latin2') then
                      CSCodePage:=Latin2
                    else if S2=UpString('Kamenicky') then
                      CSCodePage:=Kamenicky
                    else if S2=UpString('Win1250') then
                      CSCodePage:=Win1250
                    else
                      WriteErr;
                  end
                else
                  begin
                    Val(S2,LX,Code);
                    if (Length(S1)=0) or (Code<>0) or (LX<0) or (LX>65535) then
                      WriteErr
                    else
                      begin
                        if S1 = UpString('PortLPT') then
                          PortLPT:=LX
                        else if S1 = UpString('UseKeybRemoteCodes') then
                          begin
                            if LX = 0 then
                              UseKeybRemoteCodes:=False
                            else if LX=1 then
                              UseKeybRemoteCodes:=True
                            else
                              WriteErr;
                          end
                        else if S1 = UpString('CommandDelay') then
                          CommandDelay:=LX
                        else if S1 = UpString('StopDelay') then
                          StopDelay:=LX
                        else if S1 = UpString('SelectTrackDelay') then
                          SelectTrackDelay:=LX
                        else if S1 = UpString('SetNameModeDelay') then
                          SetNameModeDelay:=LX
                        else if S1 = UpString('ReSetNameModeDelay') then
                          ReSetNameModeDelay:=LX
                        else if S1 = UpString('ChangeTitleModeDelay') then
                          ChangeTitleModeDelay:=LX
                        else
                          WriteErr;
                      end;
                  end;
              end;
          end
      end;
    if ErrDetected then
      WriteLine;
    Close(F);
  end;

function ValidLine(S:String; var TrackNo:Byte; var Title:String):Boolean;
  var
    I:Integer;
    X:LongInt;
    Code:Integer;
    S1,S2:String;
  begin
    {$B-}
    S:=TrimSpace(S,All);
    if (S='') or (S[1]=';') then
      ValidLine:=False
    else
      begin
        ValidLine:=True;
        I:=Pos(')',S);
        if (I<2) or (I>4) then
          I:=Pos('.',S);
        if (I<2) or (I>4) then
          begin
            TrackNo:=0;     {MD Title}
            Title:=S;
          end
        else
          begin
            S1:=TrimSpace(Copy(S,1,I-1),All);
            S2:=TrimSpace(Copy(S,I+1,255),All);
            Val(S1,X,Code);
            if (Code=0) and (X>0) and (X<256) then
              begin
                TrackNo:=X;
                Title:=S2;
              end
            else
              begin
                TrackNo:=0;     {MD Title}
                Title:=S;
              end;
          end;
      end;
  end;

procedure SetTrack(TrackNo:Byte);
  var
    XS:String[10];
    I,XI:Byte;
    Code:Integer;
  procedure SetNumber(X:Byte);
    begin
      case X of
        1 : SendCode(ir_SonyMD_1);
        2 : SendCode(ir_SonyMD_2);
        3 : SendCode(ir_SonyMD_3);
        4 : SendCode(ir_SonyMD_4);
        5 : SendCode(ir_SonyMD_5);
        6 : SendCode(ir_SonyMD_6);
        7 : SendCode(ir_SonyMD_7);
        8 : SendCode(ir_SonyMD_8);
        9 : SendCode(ir_SonyMD_9);
       10 : SendCode(ir_SonyMD_10);
       11 : SendCode(ir_SonyMD_11);
       12 : SendCode(ir_SonyMD_12);
       13 : SendCode(ir_SonyMD_13);
       14 : SendCode(ir_SonyMD_14);
       15 : SendCode(ir_SonyMD_15);
       16 : SendCode(ir_SonyMD_16);
       17 : SendCode(ir_SonyMD_17);
       18 : SendCode(ir_SonyMD_18);
       19 : SendCode(ir_SonyMD_19);
       20 : SendCode(ir_SonyMD_20);
       21 : SendCode(ir_SonyMD_21);
       22 : SendCode(ir_SonyMD_22);
       23 : SendCode(ir_SonyMD_23);
       24 : SendCode(ir_SonyMD_24);
       25 : SendCode(ir_SonyMD_25);
     end;
    end;  {SetNumber}
  begin {SetTrack}
    if TrackNo<=25 then
      SetNumber(TrackNo)
    else
      begin
        Str(TrackNo,XS);
        SendCode(ir_SonyMD_More25);
        if Length(XS)=3 then
          SendCode(ir_SonyMD_More25);
        for I:=1 to Length(XS) do
          begin
            Val(XS[I],XI,Code);
            SetNumber(XI);
          end;
      end;
  end; {SetTrack}


begin
  WriteLn;
  WriteLn;
  WriteLine;
  WriteLn('                                 TitleMD v 2.0');
  WriteLine;
  if ParamCount<>1 then
    begin
      WriteLn('Parameters: TitleMD <SongTitlesFile>');
      WriteLn('  Structure of SongTitlesFile:');
      WriteLn('    <MiniDisc Title>');
      WriteLn('    [1)] <Title of 1. track>');
      WriteLn('    [2)] <Title of 2. track>');
      WriteLn('    [3)] <Title of 3. track>');
      WriteLn('    ...');
      WriteLn('  Empty lines and lines started with ; are comments (ignored)');
      WriteLine;
      Halt;
    end;
  ReadIniFile;
  GetConvertTable(MyConvertTable,CSCodePage,USA);
  {$I-}
  Assign(F,ParamStr(1));
  Reset(F);
   if IOResult<>0 then
     begin
       WriteLn('Error - File "'+ParamStr(1)+'" not found.');
       WriteLine;
       Halt;
     end;
  {$I+}
  WriteLn('Sending data to MiniDisc (press <Esc> to exit)');
  WriteLine;
  LastTrackNo:=-1;
  while not EOF(F) do
    begin
      ReadLn(F,S);
      S:=ConvTab2Space(S);
      S:=TrimSpace(S,All);
      for I:=1 to Length(S) do
        S[I]:=MyConvertTable[S[I]];
      if ValidLine(S,TrackNo,Title) then
        begin
          if TrackNo=0 then
            TrackNo:=LastTrackNo+1;
          if TrackNo=0 then
            Write(' MD) ')
          else
            Write(TrackNo:3,') ');
          SendCode(ir_SonyMD_Stop);
          Delay(StopDelay);
          if TrackNo>0 then
            begin
              SetTrack(TrackNo);
              Delay(SelectTrackDelay);
              SendCode(ir_SonyMD_Pause);
            end;
          SendCode(ir_SonyMD_Name);
          Delay(SetNameModeDelay);
          if UseKeybRemoteCodes then
            SendStr2MD_20bit(Title)
          else
            SendStr2MD(Title);
          SendCode(ir_SonyMD_Name);
          Delay(ReSetNameModeDelay);
          WriteLn;
          LastTrackNo:=TrackNo;
        end;
    end;
  SendCode(ir_SonyMD_Stop);
  Delay(StopDelay);
  WriteLine;
  Close(F);
end.
